# Creating The Sym Parikh Artifact

## Requirements

* [QEmu][qemu] (version used: 8.1.1)
* [Arch Linux Basic QCow image][archlinuxqcow] (version used: 2023-10-01)

## Creation Steps

Rename the image from the default and boot using QEmu.

    $ mv \
        ~/Downloads/Arch-Linux-x86_64-basic-20231001.182377.qcow2 \
        sym-parikh-img.qcow2
    $ qemu-system-x86_64 \
        -m 4G \
        --enable-kvm \
        sym-parikh-img.qcow2

Login (in QEmu virtual machine):

    archlinux login: arch
    password arch

From within QEmu virtual machine, update the install (done: 2023-10-07). Refreshing keys took a long time.

    $ sudo pacman-key --populate archlinux
    $ sudo pacman -Syu

Getting the keys up to date can be tricky, you might also need

    $ sudo pacman-key --refresh-keys

Then install required dependencies

    $ sudo pacman -S clang cmake fakeroot git less make z3

Catch2v3 needs to be installed from the Arch User Repository

    $ mkdir aur
    $ cd aur
    $ curl \
        https://aur.archlinux.org/cgit/aur.git/snapshot/catch2v3.tar.gz \
        > catch2v3.tar.gz
    $ tar xf catch2v3.tar.gz
    $ cd catch2v3
    $ makepkg -i

If you get an unknown key error, you can use `gpg --recv-keys`

    ...
    catch2v3 git repo ... FAILED (unknown public key DE48307B8B0D381A)
    $ gpg --recv-keys DE48307B8B0D381A
    $ makepkg -i

Return to the home directory

    $ cd

Clone the Sym Parikh repository

    $ git clone https://gitlab.cim.rhul.ac.uk/uxac009/symparikh.git

Build Sym Parikh

    $ cd symparikh
    $ mkdir build
    $ cd build
    $ cmake ../
    $ make

You can check the build with

    $ ./test

Or run with

    $ ./sym_parikh --file ../examples/basic_regex.smt2

Finally, place `README.md` in the root folder

    $ cd
    $ cp symparikh/artifact/README.md ./

## Benchmark Recreation

### Install Comparison Solvers

`z3` was installed above as a dependency of Sym Parikh.

`cvc5` can be installed from the AUR:

    $ cd
    $ cd aur
    $ curl \
        https://aur.archlinux.org/cgit/aur.git/snapshot/cvc5-bin.tar.gz \
        > cvc5-bin.tar.gz
    $ tar xf cvc5-bin.tar.gz
    $ cd cvc5-bin.tar.gz
    $ makepkg -i

`ostrich` has to be built from source (pick the 1st java runtime)

    $ sudo pacman -S sbt jdk17-openjdk
    $ sudo archlinux-java set java-17-openjdk
    $ cd
    $ git clone https://github.com/uuverifiers/ostrich.git
    $ cd ostrich
    $ git fetch
    $ git checkout ce855e26
    $ sbt assembly

This provides both `ostrich` and `ostrich-cea`.

Netcat is also needed

    $ pacman -S gnu-netcat

### Setup Benchmark Scripts

The `test-solvers.py` script in the `symparikh/benchmark-scripts` directory will run all solvers on a specified benchmark folder. It needs configuring to match the artifact environment.

You'll need a text editor

    $ sudo pacman -S neovim
    $ cd ~/symparikh/benchmark-scripts
    $ nvim test-solvers.py

Then

* Set the `SYM_PARIKH` variable to `/home/arch/symparikh/build/sym_parikh`.
* In `SOLVERS`
    * change the path of `ostrich` to `/home/arch/ostrich/ostrich-client`,
    * change the path of `ostrich-cea` to `/home/arch/ostrich/ostrich-cea`.

To run a set of benchmarks, install python

    $ sudo pacman -S python

Then to run, e.g.

    $ python test-solvers.py /home/arch/symparikh/benchmark-scripts/qf_slia/QF_SLIA/2015-Norn

To recreate the graphs, Pandas is needed

    $ sudo pacman -S python-pandas

### Text Editors

Install some popular text editors

    $ sudo pacman -S neovim nano emacs

### Unused scripts

Remove `benchmark-scripts/get-regexlib.py` -- i needs libraries that are awkward to install, and isn't really needed for the artifact.

[archlinuxqcow]: https://gitlab.archlinux.org/archlinux/arch-boxes/
[qemu]: https://www.qemu.org/
