
# POPL 2024 Artifact

Welcome to the POPL 2024 Sym Parikh artifact.

See `README-artifact-creation.md` for how to build the artifact.

## Requirements

* [QEmu][qemu] (version used: 8.1.1)
* Sym Parikh artifact QCow image (`sym-parikh-img.qcow2`)

## Running the Image

Start the image using QEmu. The command below works on many systems, but the exact command may depend on the machine you are using. Please see the QEmu documentation for more details. The `--enable-kvm` flag should be removed if your system does not support KVM.

    $ qemu-system-x86_64 \
        -m 4G \
        --enable-kvm \
        sym-parikh-img.qcow2

Once booted, login as the default user

    archlinux login: arch
    password arch

## Getting Started in the Image

The image provides a command line in an Arch Linux system. This README is in the home directory. To view files you can use some standard facilities.

    $ more README.md
    $ less README.md

Neovim, Emacs, and Nano are also installed

    $ nvim README.md
    $ emacs README.md
    $ nano README.md

## Running Sym Parikh in the Image

Enter the `symparikh` directory

    $ cd symparikh

You can do a basic run of the tool with

    $ ./build/sym_parikh --file examples/basic-regex.smt2

Use `--help` for available flags.

    $ ./build/sym_parikh --help

## Recreating the Benchmarks

The image comes with Z3, CVC5, and two variants of Ostrich. Z3 and CVC5 are installed system wide and can be run with the commands `z3` and `cvc5`. Ostrich has two variants in the folder `/home/arch/ostrich` using two scripts `ostrich-client` and `ostrich-cea`.

In `/home/arch/symparikh/benchmark-scripts` there is a script `test-solvers.py`. The benchmarks are also available in the same directory, under `qf_slia` and `wordeqs`.

To recreate the experimental data

    $ cd ~/symparikh/benchmark-scripts
    $ mkdir results
    $ python test-solvers.py qf_slia/QF_SLIA/2015-Norn > results/norn.csv
    $ python test-solvers.py qf_slia/QF_SLIA/2018-Keplar > results/keplar.csv
    $ python test-solvers.py wordeqs > results/wordeqs.csv

Then to recreate the LaTeX for the graphs

    $ python make-graphs.py results/*.csv

The data files and LaTeX files are written to the `processed-results` directory.

Inside the image `results-2023-07-05` is the original data for the submitted paper.

[qemu]: https://www.qemu.org/
