
import random

from datetime import datetime
from typing import List

# Adjustable
EQ_LEN_MIN = 3
EQ_LEN_MAX = 3
NUM_PREDICATES_MIN = 1
NUM_PREDICATES_MAX = 3
NUM_INSTANCES = 100
FILENAME_FORMAT = "wordeq_{}.smt2"

# Semi adjustable
TODAY = datetime.now().strftime("%Y-%m-%d")

# Not adjustable
NUM_VARS = 3

def get_var(idx : int) -> str:
    return f"x{idx}"

def esc_chr(chr : int) -> str:
    return "\\u{" + f"{chr:x}" + "}"

def make_permutation(eq_len : int) -> List[str]:
    return [
        get_var(random.randint(0, NUM_VARS - 1))
        for i in range(eq_len)
    ]

def make_sub_re() -> str:
    lo = random.randint(0, 255)
    hi = random.randint(lo, 255)
    return f"(re.range \"{esc_chr(lo)}\" \"{esc_chr(hi)}\")"

def make_pred_seq(length : int) -> List[str]:
    return [ make_sub_re() for i in range(length) ]

for i in range(NUM_INSTANCES):
    eq_len = random.randint(EQ_LEN_MIN, EQ_LEN_MAX)

    all_vars = set()

    # generate permutations until all vars used
    while len(all_vars) < NUM_VARS:
        lhs = make_permutation(eq_len)
        rhs = make_permutation(eq_len)
        all_vars = set(lhs + rhs)

    num_lpreds = random.randint(NUM_PREDICATES_MIN, NUM_PREDICATES_MAX)
    num_rpreds = random.randint(NUM_PREDICATES_MIN, NUM_PREDICATES_MAX)

    lpreds = make_pred_seq(num_lpreds)
    rpreds = make_pred_seq(num_lpreds)

    with open(FILENAME_FORMAT.format(i), "w") as f:
        f.write(
            "(set-info :smt-lib-version 2.6)\n" +
            "(set-logic QF_S)\n" +
            "(set-info :source |\n" +
            f"Generated by: for sym_parikh benchmarks" +
            f"Generated on: {TODAY}\n" +
            "Generator: sym_parikh\n" +
            "Application: Parikh-image abstraction for string analysis\n" +
            "Target solver: sym_parikh\n" +
            "Publication: TBC\n" +
            f"Description: randomly generated\n" +
            "|)\n" +
            "".join(
                f"(declare-const {get_var(j)} String)\n"
                for j in range(NUM_VARS)
            ) +
            f"(assert (= (str.++ {' '.join(lhs)}) (str.++ {' '.join(rhs)})))\n" +
            f"(assert (str.in_re {get_var(0)} (re.* (re.++ " +
                " ".join(lpreds) +
                "))))\n" +
            f"(assert (str.in_re {get_var(1)} (re.++ " +
                "(re.+ (re.++ " + " ".join(lpreds) + ")) " +
                "(re.+ (re.++ " + " ".join(rpreds) + "))" +
                ")))\n" +
            f"(assert (str.in_re {get_var(2)} (re.* (re.++ " +
                " ".join(rpreds) +
                "))))\n" +
            "(check-sat)"
        )
