
# Running In Docker

This Docker image may be used to help run the tool in environments where the Z3 or Catch2v3 dependencies are not working correctly.

You will need

* [Docker][docker]
* [Docker Buildx][docker-buildx]

The instructions below assume you are running from the command line.

## Building the Image

First build the image. Make sure you are on the command line in the directory containing the `Dockerfile`. That's the `docker` directory where this README resides.

Then run

    $ docker buildx build --tag local/sym_parikh .

In the above command `--tag` gives the image the name `local/sym_parikh` for later reference. The final `.` is the path to folder containing the Dockerfile (i.e. the current directory).

## Running the Image

First cd to the repo root directory. If you just built the Docker file, you need to go back one.

    $ cd ..

The image will provide a Bash command line in an (Arch Linux) environment where `sym_parikh` can be compiled. We use the following command, in shorthand. The full command is explained below.

    $ docker run --rm -it -v ./:/var/cwd local/sym_parikh

In longhand, this command is

    $ docker run \
        --rm \                  # * Remove the container when done
        --interactive \         # * Enable you to type commands in the container
        --tty \                 # * Enable you to view the command line output
        --volume ./:/var/cwd \  # * Make the project directory (./) available
                                #   on the path /var/cwd in the container
        local/sym_parikh        # * The tag of the image we built

You can run the command from anywhere as long as you replace `./` with the path to the `sym_parikh` repository.

You should get a Bash prompt. E.g.

    [build@f0e913595927 cwd]$


## Running Sym\_Parikh

Once Bash is running, the current working directory should be `/var/cwd`, which is mapped to the directory containing `sym_parikh` on your machine. You can make sure things are ok with an `ls`

    [build@f0e913595927 cwd] $ ls
    CMakeLists.txt
    README.md
    src
    ...

Then to build, follow the standard CMake process. I would use a dedicated Docker build directory so the binaries that only work in this container are quarantined.

    $ mkdir build-docker
    $ cd build-docker
    $ cmake ../
    $ make

Then run

    $ ./sym_parikh --file ../examples/basic-regex.smt2

Note, any file you want to test the code on needs to be available in the Docker container. The easiest way to do this is to put it in a subdirectory of the `sym_parikh` repo. You can also make other external volumes available with the `--volumes` flag to the Docker run command.

[docker]: https://www.docker.com/
[docker-buildx]: https://docs.docker.com/engine/reference/commandline/buildx/
