
#include <getopt.h>
#include <string.h>

#include "command_line.h"

using namespace std;
using namespace sym_parikh;

CommandLine::CommandLine(int argc, char** argv)
    : pi_provider(new ParikhImageProviderOne()),
        check_compatibility(false),
        is_help(false),
        print_predicates(false),
        preds_from_lits(false),
        get_model(false),
        predicate_num_limit(-1) {
    static struct option long_options[] = {
        {"check-compat", no_argument, 0,  'c'},
        {"file", required_argument, 0,  'f'},
        {"get-model", no_argument, 0,  'm'},
        {"parikh-image", required_argument, 0,  'i'},
        {"predicate-num-limit", required_argument, 0,  'n'},
        {"predicates-file", required_argument, 0,  'p'},
        {"preds-from-lits", no_argument, 0,  'l'},
        {"print-predicates", no_argument, 0,  'x'},
        {"help", no_argument, 0,  'h'},
        {0, 0, 0, 0}
    };

    // reset to start in case of multiple calls
    optind = 1;

    while (true) {
        int c = getopt_long(argc, argv, "cf:i:hlmn:p:x", long_options, NULL);
        if (c == -1)
            break;

        switch (c) {
        case 'c':
            check_compatibility = true;
            break;
        case 'h':
            is_help = true;
            break;
        case 'i':
            if (strcmp(optarg, "2") == 0) {
                pi_provider = std::shared_ptr<ParikhImageProvider>(
                    new ParikhImageProviderTwo()
                );
            }
            // else keep provider 1
            break;
        case 'l':
            preds_from_lits = true;
            break;
        case 'm':
            get_model = true;
            break;
        case 'n': {
            string count(optarg);
            predicate_num_limit = stoi(count);
            break;
        }
        case 'f':
            smt_filename.reset(new string(optarg));
            break;
        case 'p':
            predicates_filename.reset(new string(optarg));
            break;
        case 'x':
            print_predicates = true;
            break;
        }
    }
}

string CommandLine::get_usage() const {
    return string(
        "Usage: sym_parikh --file <smt_filename> --parikh-image <n> \n"
        "\n"
        "Args:\n"
        "\n"
        "  -c --check-compat don't solve, but check if file is compatible\n"
        "  -f --file <smt_filename> file containing the SMT regex to parse\n"
        "  -i --parikh-image <n> which parikh image method to use (def: 1)\n"
        "  -n --predicate-num-limit <n> maximum number of predicates to take from formula\n"
        "  -p --predicates-file <filename> an SMT file containing a sequence of assertions over a single free variable x\n"
        "  -l --preds-from-lits get predicates from string literals as well as regular expressions (by default only get preds from lits if none in regexp)"
        "  -x --print-predicates print out the predicates used in the analysis\n"
    );
}
