#ifndef COMMAND_LINE_H
#define COMMAND_LINE_H

#include <memory>
#include <string>

#include "parikh_image_provider.h"

using namespace std;

namespace sym_parikh {
    class CommandLine {
        public:
        CommandLine(int argc, char **argv);

        bool get_is_help() const { return is_help; }
        bool get_check_compatibility() const { return check_compatibility; }
        shared_ptr<string> get_smt_filename() const { return smt_filename; }
        bool get_print_predicates() const { return print_predicates; }
        bool get_preds_from_lits() const { return preds_from_lits; }
        bool get_get_model() const { return get_model; }
        int get_predicate_num_limit() const { return predicate_num_limit; }

        string get_usage() const;

        std::shared_ptr<ParikhImageProvider> get_pi_provider() const {
            return pi_provider;
        }

        /**
         * User specified predicates, or empty if none
         *
         * Predicates should be an SMT file of the form
         * ...
         * (assert (= x "a"))
         * (assert (str.in_re x (re.range "a" "b")))
         * ...
         * I.e. assertions over a single free variable x.
         */
        shared_ptr<string> get_predicates_filename() const {
            return predicates_filename;
        }

        private:
        shared_ptr<string> smt_filename;
        bool is_help;
        bool check_compatibility;
        std::shared_ptr<ParikhImageProvider> pi_provider;
        shared_ptr<string> predicates_filename;
        bool print_predicates;
        bool preds_from_lits;
        bool get_model;
        int predicate_num_limit;
    };
}

#endif
