
#ifndef __PARIKH_IMAGE_H__
#define __PARIKH_IMAGE_H__

#include "z3++.h"

#include "re2automaton.h"
#include "symbolic_automaton.h"

namespace sym_parikh {
    /**
     * Represents the Parikh image of a regular expression
     */
    class ParikhImage {
        public:
            /**
             * Base ParikhImage
             *
             * @param context the z3 context to use
             * @param variable_prefix prefix to add to variable names to make
             * sure they're unique
             */
            ParikhImage(z3::context &context) : context(context) { }

            /**
             * Get the Parikh image formula
             */
            virtual z3::expr const get_image_formula() const = 0;

            /**
             * Get the automaton the Parikh image came from
             */
            virtual std::shared_ptr<symbolic_automaton const>
            get_re_automaton() = 0;

        protected:
            z3::context &context;

            /**
             * Helper function for subclasses
             *
             * To make it easier to manage their list of internal variables
             *
             * @param map the map to read from
             * @param variables load variables into this vector
             */
            template <typename T> void get_variables_from_map(
                std::map<T, z3::expr> const &map,
                z3::expr_vector &variables
            ) {
                for (auto const &kv : map)
                    variables.push_back(kv.second);
            }

            z3::expr make_char_var(std::string const &name) {
                return context.int_const(name.c_str());
            }

            z3::expr get_free_variable_var() {
                return re2automaton::get_free_variable(context);
            }
    };
}

#endif
