
#ifndef __PARIKH_IMAGE_PROVIDER_H__
#define __PARIKH_IMAGE_PROVIDER_H__

#include "z3++.h"

#include "parikh_image.h"
#include "parikh_image_one.h"
#include "parikh_image_two.h"
#include "re2automaton.h"
#include "symbolic_automaton.h"

namespace sym_parikh {
    class ParikhImageProvider {
        public:
            virtual std::shared_ptr<ParikhImage> get_parikh_image(
                z3::context &context,
                std::shared_ptr<symbolic_automaton> const &automaton,
                std::vector<z3::expr> const &predicates,
                std::vector<z3::expr> const &pred_vars
            ) = 0;

            std::shared_ptr<ParikhImage> get_parikh_image(
                z3::context &context,
                z3::expr const &source_re,
                std::vector<z3::expr> const &predicates,
                std::vector<z3::expr> const &pred_vars
            ) {
                return get_parikh_image(
                    context,
                    get_automaton(context, source_re),
                    predicates,
                    pred_vars
                );
            }

        private:
            std::shared_ptr<symbolic_automaton> get_automaton(
                z3::context &context,
                z3::expr const &source_re
            ) {
                re2automaton re2aut(context);
                return re2aut.translate(source_re);
            }
    };

    class ParikhImageProviderOne : public ParikhImageProvider {
        public:
            using ParikhImageProvider::get_parikh_image;

            virtual std::shared_ptr<ParikhImage> get_parikh_image(
                z3::context &context,
                std::shared_ptr<symbolic_automaton> const &automaton,
                std::vector<z3::expr> const &predicates,
                std::vector<z3::expr> const &pred_vars
            ) {
                return std::shared_ptr<ParikhImage>(new ParikhImageOne(
                    context,
                    automaton,
                    predicates,
                    pred_vars
                ));
            }
    };

    class ParikhImageProviderTwo : public ParikhImageProvider {
        public:
            using ParikhImageProvider::get_parikh_image;

            virtual std::shared_ptr<ParikhImage> get_parikh_image(
                z3::context &context,
                std::shared_ptr<symbolic_automaton> const &automaton,
                std::vector<z3::expr> const &predicates,
                std::vector<z3::expr> const &pred_vars
            ) {
                return std::shared_ptr<ParikhImage>(new ParikhImageTwo(
                    context,
                    automaton,
                    predicates,
                    pred_vars
                ));
            }
    };
}

#endif
