
#ifndef __REGEX_H__
#define __REGEX_H__

// for easy debugging
#include <iostream>
#include <vector>

#include "z3++.h"

#include "app.h"

namespace sym_parikh {
    /**
     * Helper functions for navigating z3 regex
     */
    class regex : public app {
        public:
        /**
         * str.to_re arg0
         */
        static bool is_to_re(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_SEQ_TO_RE, args);
        }

        /**
         * re.concat arg0 arg1
         */
        static bool is_concat(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_CONCAT, args);
        }

        /**
         * re.union arg0 arg1
         */
        static bool is_union(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_UNION, args);
        }

        /**
         * re.star arg0
         */
        static bool is_star(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_STAR, args);
        }

        /**
         * re.plus arg0
         */
        static bool is_plus(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_PLUS, args);
        }

        /**
         * re.opt arg0
         */
        static bool is_opt(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_OPTION, args);
        }

        /**
         * re.range
         */
        static bool is_range(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_RANGE, args);
        }

        /**
         * re.not arg0
         */
        static bool is_complement(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_RE_COMPLEMENT, args);
        }

        /**
         * re.loop arg0 lo hi
         */
        static bool is_loop(
            z3::expr const &e,
            std::vector<z3::expr> &body_arg,
            unsigned& lo,
            unsigned& hi
        ) {
            if (e.decl().decl_kind() == Z3_OP_RE_LOOP && e.hi() > 0) {
                body_arg.push_back(e.arg(0));
                lo = e.lo();
                hi = e.hi();
                return true;
            } else {
                return false;
            }
        }

        /**
         * re.loop arg0 lo
         */
        static bool is_loop(
            z3::expr const &e,
            std::vector<z3::expr> &body_arg,
            unsigned& lo
        ) {
            if (e.decl().decl_kind() == Z3_OP_RE_LOOP && e.hi() == 0) {
                body_arg.push_back(e.arg(0));
                lo = e.lo();
                return true;
            } else {
                return false;
            }
        }

        /**
         * re.empty
         */
        static bool is_empty(z3::expr const &e) {
            // TODO: check this is right op
            return is_app_of(e, Z3_OP_SEQ_EMPTY);
        }

        /**
         * sigma*
         */
        static bool is_full_seq(z3::expr const &e) {
            // TODO: check this is right op
            return is_app_of(e, Z3_OP_RE_FULL_SET);
        }

        /**
         * sigma
         */
        static bool is_full_char(z3::expr const &e) {
            // TODO: check this is right op
            return is_app_of(e, Z3_OP_RE_FULL_CHAR_SET);
        }

        /**
         * re.and arg0 arg1
         */
        static bool is_intersection(
            z3::expr const &e,
            std::vector<z3::expr> &args
        ) {
            return is_app_of(e, Z3_OP_RE_INTERSECT, args);
        }

        /**
         * re.none
         */
        static bool is_none(z3::expr const &e) {
            return is_app_of(e, Z3_OP_RE_EMPTY_SET);
        }
    };
}

#endif
