
#ifndef __ZSTRING_H__
#define __ZSTRING_H__

#include <vector>

#include "z3++.h"

#include "app.h"

namespace sym_parikh {
    /**
     * Helper functions for navigating z3 regex
     */
    class zstring : public app {
        public:
        static bool is_string(z3::expr const &e, std::string &value) {
            if (e.is_string_value()) {
                value = e.get_string();
                return true;
            } else {
                return false;
            }
        }

        /**
         * I think, through the z3 API, chars are len 1 strings
         */
        static bool is_unit_char(z3::expr const &e, std::string &value) {
            if (e.is_string_value() && e.get_string().size() == 1) {
                value = e.get_string();
                return true;
            } else {
                return false;
            }
        }

        static bool is_concat(
            z3::expr const &e,
            std::vector<z3::expr> &args
        ) {
            return is_app_of(e, Z3_OP_SEQ_CONCAT, args);
        }

        static bool is_unit(z3::expr const &e, std::vector<z3::expr> &args) {
            return is_app_of(e, Z3_OP_SEQ_UNIT, args);
        }

        static bool is_empty(z3::expr const &e) {
            return is_app_of(e, Z3_OP_SEQ_EMPTY);
        }

        static bool is_param(z3::expr const &e) {
            return e.is_const() && !e.is_string_value();
        }
    };
}

#endif
