
#include <iostream>

#include <catch2/catch_test_macros.hpp>

#include "z3++.h"

#include "symbolic_automaton.h"

using namespace std;
using namespace sym_parikh;
using namespace z3;

TEST_CASE("Empty automaton", "[automata-empty]") {
    symbolic_automaton aut;
    REQUIRE(aut.is_empty());
}

TEST_CASE("Singleton", "[automata-singleton]") {
    context c;
    expr x = c.int_const("x");
    expr lbl = x > 10;
    vector<expr> seq = { lbl };
    symbolic_automaton aut(seq);
    REQUIRE(aut.num_states() == 2);
}

TEST_CASE("Concat", "[automata-concat]") {
    context c;
    expr x = c.int_const("x");

    expr lbl1 = x > 10;
    vector<expr> seq1 = { lbl1 };
    symbolic_automaton aut1(seq1);

    expr lbl2 = x > 15;
    vector<expr> seq2 = { lbl2 };
    symbolic_automaton aut2(seq2);

    unique_ptr<symbolic_automaton> concat(
        symbolic_automaton::mk_concat(aut1, aut2)
    );
    concat->compress();

    REQUIRE(concat->move_count() == 2);
}
