
#include <iostream>

#include <catch2/catch_test_macros.hpp>

#include "command_line.h"

using namespace sym_parikh;

TEST_CASE("Help is registered", "[command-line-help]") {
    char dummy[] = "dummy";
    char help[] = "-h";
    char *argv[] =  { dummy, help };
    CommandLine command_line(2, argv);
    REQUIRE(command_line.get_is_help());
}

TEST_CASE("No filename given", "[command-line-filename-none]") {
    char dummy[] = "dummy";
    char file[] = "--fail";
    char filename[] = "testfilename.smt2";
    char *argv[] =  { dummy, file, filename };
    CommandLine command_line(3, argv);
    REQUIRE(command_line.get_smt_filename() == NULL);
}

TEST_CASE("Filename given", "[command-line-filename]") {
    char dummy[] = "dummy";
    char file[] = "--file";
    char filename[] = "testfilename.smt2";
    char *argv[] =  { dummy, file, filename };
    CommandLine command_line(3, argv);
    REQUIRE(*command_line.get_smt_filename() == "testfilename.smt2");
}

TEST_CASE("Preds from lits", "[command-line-preds-from-lits]") {
    char dummy[] = "dummy";
    char file[] = "--file";
    char filename[] = "testfilename.smt2";
    char preds_from_lits[] = "--preds-from-lits";
    char *argv[] =  { dummy, preds_from_lits, file, filename };
    CommandLine command_line(4, argv);
    REQUIRE(command_line.get_preds_from_lits());
}

TEST_CASE("Preds from lits false", "[command-line-preds-from-lits]") {
    char dummy[] = "dummy";
    char file[] = "--file";
    char filename[] = "testfilename.smt2";
    char *argv[] =  { dummy, file, filename };
    CommandLine command_line(3, argv);
    REQUIRE(!command_line.get_preds_from_lits());
}

TEST_CASE("Predicate num limit", "[command-line-pred-num-limit]") {
    char dummy[] = "dummy";
    char file[] = "--file";
    char filename[] = "testfilename.smt2";
    char predicate_num_limit[] = "--predicate-num-limit";
    char predicate_num_limit_n[] = "4";
    char *argv[] =  {
        dummy,
        predicate_num_limit,
        predicate_num_limit_n,
        file,
        filename
    };
    CommandLine command_line(4, argv);
    REQUIRE(command_line.get_predicate_num_limit() == 4);
}

TEST_CASE("Predicate num limit false", "[command-line-pred-num-limit-false]") {
    char dummy[] = "dummy";
    char file[] = "--file";
    char filename[] = "testfilename.smt2";
    char *argv[] =  { dummy, file, filename };
    CommandLine command_line(3, argv);
    REQUIRE(command_line.get_predicate_num_limit() < 0);
}


