
// for easy debuggin
#include <iostream>
#include <set>

#include <catch2/catch_test_macros.hpp>

#include "z3++.h"

#include "config.h"
#include "parikh_profile.h"
#include "re2automaton.h"
#include "util.h"

using namespace std;
using namespace sym_parikh;
using namespace z3;

static bool same_z3_exprs(vector<expr> const &es1, vector<expr> const &es2);
static expr smtlib2expr(
    context &context,
    string const &var_decls,
    string const &assertion
);

TEST_CASE("Norn profile", "[parikh-profile-norn]") {
    context c;

    const expr_vector &v = c.parse_file(
        SOURCE_DIR "/src/test/examples/norn-benchmark-26.smt2"
    );
    ParikhProfile profile(c, v);

    vector<expr> exp_seq_vars;
    exp_seq_vars.push_back(c.string_const("var_5"));
    exp_seq_vars.push_back(c.string_const("var_8"));
    exp_seq_vars.push_back(c.string_const("var_10"));
    exp_seq_vars.push_back(c.string_const("var_11"));
    exp_seq_vars.push_back(c.string_const("var_12"));
    REQUIRE(same_z3_exprs(exp_seq_vars, profile.get_seq_vars()));

    string var_decls =
        "(declare-fun var_5 () String)"
        "(declare-fun var_8 () String)"
        "(declare-fun var_10 () String)"
        "(declare-fun var_11 () String)"
        "(declare-fun var_12 () String)";

    vector<expr> exp_re_checks;
    vector<expr> exp_not_re_checks;
    exp_re_checks.push_back(smtlib2expr(
        c,
        var_decls,
        "(let ((a!1 (re.union (str.to_re \"z\")"
            "(re.union (re.union (str.to_re \"a\") (str.to_re \"b\"))"
                                 "(str.to_re \"c\")))))"
            "(str.in_re (str.++ var_10 var_11 var_12 \"z\" var_10) (re.* a!1)))"
    ));
    exp_re_checks.push_back(smtlib2expr(
        c,
        var_decls,
        "(str.in_re var_10 (re.* (re.range \"a\" \"u\")))"
    ));
    exp_re_checks.push_back(smtlib2expr(
        c,
        var_decls,
        "(str.in_re (str.++ var_10 var_11 var_12)"
                    "(re.* (re.range \"a\" \"u\")))"
    ));
    exp_not_re_checks.push_back(smtlib2expr(
        c,
        var_decls,
        "(let ((a!1 (re.union (str.to_re \"z\")"
                     "(re.union (re.union (str.to_re \"a\")"
                                          "(str.to_re \"b\"))"
                               "(str.to_re \"c\")))))"
              "(not (str.in_re"
                        "(str.++ var_10 var_11 \"c\" var_12 \"z\" var_10 var_11)"
                        "(re.* a!1))))"
    ));

    REQUIRE(same_z3_exprs(exp_re_checks, profile.get_in_re_checks()));
    REQUIRE(same_z3_exprs(exp_not_re_checks, profile.get_not_in_re_checks()));

    vector<expr> exp_lens;
    exp_lens.push_back(smtlib2expr(c, var_decls, "(str.len var_5)"));
    exp_lens.push_back(smtlib2expr(c, var_decls, "(str.len var_8)"));
    exp_lens.push_back(smtlib2expr(c, var_decls, "(str.len var_10)"));
    exp_lens.push_back(smtlib2expr(c, var_decls, "(str.len var_11)"));
    exp_lens.push_back(smtlib2expr(c, var_decls, "(str.len var_12)"));
    REQUIRE(same_z3_exprs(exp_lens, profile.get_seq_lens()));

    re2automaton re2aut(c);

    expr t = c.bool_val(true);

    vector<expr> exp_preds;
    exp_preds.push_back(re2aut.get_is_char('z'));
    exp_preds.push_back(re2aut.get_is_char('c'));
    exp_preds.push_back(re2aut.get_is_char('b'));
    exp_preds.push_back(re2aut.get_is_char('a'));
    exp_preds.push_back(re2aut.get_in_range('a', 'u'));
    exp_preds.push_back(
        t &&
        re2aut.get_is_char('z') &&
        !re2aut.get_is_char('c') &&
        !re2aut.get_is_char('a') &&
        !re2aut.get_is_char('b')
    );
    exp_preds.push_back(
        t &&
        !re2aut.get_is_char('z') &&
        re2aut.get_is_char('c') &&
        !re2aut.get_is_char('a') &&
        !re2aut.get_is_char('b')
    );
    exp_preds.push_back(
        t &&
        !re2aut.get_is_char('z') &&
        !re2aut.get_is_char('c') &&
        !re2aut.get_is_char('a') &&
        re2aut.get_is_char('b')
    );
    exp_preds.push_back(
        t &&
        !re2aut.get_is_char('z') &&
        !re2aut.get_is_char('c') &&
        re2aut.get_is_char('a') &&
        !re2aut.get_is_char('b')
    );
    exp_preds.push_back(
        t &&
        !re2aut.get_is_char('z') &&
        !re2aut.get_is_char('c') &&
        !re2aut.get_is_char('a') &&
        !re2aut.get_is_char('b')
    );
    exp_preds.push_back(t);

    REQUIRE(same_z3_exprs(exp_preds, profile.get_predicates()));
}

TEST_CASE("Seq eqs", "[parikh-profile-seq-eqs]") {
    context c;
    const expr_vector &v = c.parse_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (and (= x y) (= x \"a\")))\n"
        "(check-sat)\n"
    );
    ParikhProfile profile(c, v);

    expr x = c.string_const("x");
    expr y = c.string_const("y");
    expr sa = c.string_val("a");
    expr sb = c.string_val("b");
    expr sz = c.string_val("z");

    vector<expr> exp_eqs;
    exp_eqs.push_back(x == y);
    exp_eqs.push_back(x == sa);

    REQUIRE(same_z3_exprs(exp_eqs, profile.get_seq_equalities()));
}

TEST_CASE("Seq eqs extended", "[parikh-profile-seq-eqs-extended]") {
    context c;
    const expr_vector &v = c.parse_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (and (str.prefixof x y) (str.suffixof x \"z\")))\n"
        "(assert (and (str.contains x \"a\") (= y \"b\")))\n"
        "(check-sat)\n"
    );
    ParikhProfile profile(c, v);

    expr x = c.string_const("x");
    expr y = c.string_const("y");
    expr sa = c.string_val("a");
    expr sb = c.string_val("b");
    expr sz = c.string_val("z");

    vector<expr> exp_eqs;
    exp_eqs.push_back(prefixof(x, y));
    exp_eqs.push_back(x.contains(sa));
    exp_eqs.push_back(suffixof(x, sz));
    exp_eqs.push_back(y == sb);

    REQUIRE(same_z3_exprs(exp_eqs, profile.get_seq_equalities()));
}


TEST_CASE("Re eqs", "[parikh-profile-re-eqs]") {
    context c;
    const expr_vector &v = c.parse_string(
        "(assert (and (= (re.* (str.to_re \"a\")) (re.* (str.to_re \"b\")))))\n"
        "(assert (not (= (re.+ (str.to_re \"c\")) (re.* (str.to_re \"d\")))))\n"
        "(check-sat)\n"
    );
    ParikhProfile profile(c, v);

    expr sa = to_re(c.string_val("a"));
    expr sb = to_re(c.string_val("b"));
    expr sc = to_re(c.string_val("c"));
    expr sd = to_re(c.string_val("d"));

    vector<expr> exp_eqs;
    exp_eqs.push_back(star(sa) == star(sb));

    vector<expr> exp_noteqs;
    exp_noteqs.push_back(!(z3::plus(sc) == star(sd)));

    REQUIRE(same_z3_exprs(exp_eqs, profile.get_re_equalities()));
    REQUIRE(same_z3_exprs(exp_noteqs, profile.get_re_notequalities()));
}

TEST_CASE("Predicates file", "parikh-profile-predicates-file") {
    context c;

    const expr_vector &v = c.parse_file(
        SOURCE_DIR "/src/test/examples/norn-benchmark-26.smt2"
    );
    ParikhProfile profile(
        c,
        v,
        ParikhProfileConfig(
            make_shared<string>(
                SOURCE_DIR "/src/test/examples/pred_file.smt2"
            ),
            false
        )
    );

    re2automaton re2aut(c);

    vector<expr> exp_preds;
    exp_preds.push_back(re2aut.get_is_char('a'));
    exp_preds.push_back(re2aut.get_in_range('a', 'z'));
    exp_preds.push_back(
        re2aut.get_is_char('b') && re2aut.get_in_range('a', 'z')
    );

    REQUIRE(same_z3_exprs(exp_preds, profile.get_predicates()));
}

TEST_CASE("Predicates from lits", "parikh-profile-predicates-from_lits") {
    context c;

    const expr_vector &v = c.parse_string(
        "(declare-const x String)\n"
        "(assert (str.in_re (str.++ x \"a\") re.all))\n"
        "(assert (not (str.in_re (str.++ x \"b\") re.all)))\n"
        "(assert (= (str.++ x \"c\") \"d\"))"
        "(assert (= (str.++ x \"e\") \"f\"))"
    );

    ParikhProfile profile(c, v, ParikhProfileConfig(nullptr, true));

    re2automaton re2aut(c);
    vector<expr> exp_preds;

    exp_preds.push_back(re2aut.get_is_char('a'));
    exp_preds.push_back(re2aut.get_is_char('b'));
    exp_preds.push_back(re2aut.get_is_char('c'));
    exp_preds.push_back(re2aut.get_is_char('d'));
    exp_preds.push_back(re2aut.get_is_char('e'));
    exp_preds.push_back(re2aut.get_is_char('f'));
    exp_preds.push_back(c.bool_val(true));

    REQUIRE(same_z3_exprs(exp_preds, profile.get_predicates()));
}

static bool same_z3_exprs(vector<expr> const &es1, vector<expr> const &es2) {
    set<unsigned> es1_ids;
    set<unsigned> es2_ids;

    for (expr e : es1)
        es1_ids.insert(e.id());

    for (expr e : es2)
        es2_ids.insert(e.id());

    return es1_ids == es2_ids;
}

static expr smtlib2expr(
    context &context,
    string const &var_decls,
    string const &assertion
) {
    // assert (= exp exp) so that any type of expression will work
    // (hack)
    string full =
        var_decls + "(assert (= " + assertion + " " + assertion + "))\n";
    return context.parse_string(full.c_str())[0].args()[0];
}


