
#include <iostream>

#include <catch2/catch_test_macros.hpp>

#include "sym_parikh.h"

using namespace std;
using namespace sym_parikh;
using namespace z3;

TEST_CASE("Simple unsat", "[sym-parikh-simple-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Two-var unsat", "[sym-parikh-two-var-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"b\"))))\n"
        "(assert (str.in_re (str.++ y x)"
                            "(re.++ (re.+ (str.to_re \"a\"))"
                                    "(str.to_re \"c\")"
                                    "(re.+ (str.to_re \"b\")))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Simple sat", "[sym-parikh-simple-sat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re x (re.+ (re.range \"a\" \"c\"))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("One-var sat bug", "[sym-parikh-one-var-sat-bug]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re x (re.* (str.to_re \"a\"))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Two-var sat", "[sym-parikh-two-var-sat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"b\"))))\n"
        "(assert (str.in_re (str.++ x \"c\" y)"
                            "(re.++ (re.+ (str.to_re \"a\"))"
                                    "(str.to_re \"c\")"
                                    "(re.+ (str.to_re \"b\")))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Two-var sat overapprox", "[sym-parikh-two-var-sat-overapprox]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"b\"))))\n"
        "(assert (str.in_re (str.++ x \"c\" y)"
                            "(re.++ (re.+ (str.to_re \"a\"))"
                                    "(str.to_re \"c\")"
                                    "(re.+ (str.to_re \"b\")))))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Neg basic", "[neg-basic]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.* (str.to_re \"a\"))))\n"
        "(assert (not (str.in_re x (re.* (str.to_re \"a\")))))\n"
        "(check-sat)\n"
    );

    check_result res = sp.check_parikh_sat();
    REQUIRE(res == unsat);
}

TEST_CASE("Nnf basic", "[nnf-basic]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (or "
            "(str.in_re x (re.* (str.to_re \"a\"))) "
            "(str.in_re x (re.* (re.range \"b\" \"c\"))) "
        "))\n"
        "(assert (not (or "
            "(str.in_re x (re.* (str.to_re \"a\"))) "
            "(str.in_re x (re.* (re.range \"b\" \"c\"))) "
        ")))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Nnf concat", "[nnf-concat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun var_10 () String)\n"
        "(declare-fun var_11 () String)\n"
        "(assert (str.in_re (str.++ var_10 var_11 var_10) (re.* (str.to_re \"z\"))))\n"
        "(assert (not (str.in_re (str.++ var_10 var_11 ) (re.* (str.to_re \"z\")))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("String eq same", "[sym-parikh-string-eq-same]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.* (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"a\"))))\n"
        "(assert (= x y))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("String eq not same", "[sym-parikh-string-eq-not-same]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.* (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"b\"))))\n"
        "(assert (= x y))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("String concat eq same", "[sym-parikh-string-concat-eq-same]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.* (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"a\"))))\n"
        "(assert (= (str.++ x \"a\") y))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE(
    "String concat eq not same",
    "[sym-parikh-string-concat-eq-not-same]"
) {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(declare-fun y () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (str.in_re y (re.+ (str.to_re \"b\"))))\n"
        "(assert (= (str.++ x \"b\") y))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

//Currently not terminating
//TEST_CASE("Regex equal sat", "[sym-parikh-regex-eq-sat]"
//) {
//    SymParikh sp = SymParikh::from_string(
//        "(assert (= (re.* (str.to_re \"a\")) "
//                    "(re.union (str.to_re \"\") (re.+ (str.to_re \"a\")))))\n"
//    );
//    REQUIRE(sp.check_parikh_sat() == sat);
//}

TEST_CASE("Regex equal unsat", "[sym-parikh-regex-eq-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(assert (= (re.* (str.to_re \"b\")) "
                    "(re.union (str.to_re \"\") (re.+ (str.to_re \"a\")))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Regex not equal sat", "[sym-parikh-regex-neq-sat]") {
    SymParikh sp = SymParikh::from_string(
        "(assert (not (= (re.* (str.to_re \"b\")) "
                        "(re.union (str.to_re \"\") "
                                    "(re.+ (str.to_re \"a\"))))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Regex not equal unsat", "[sym-parikh-regex-neq-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(assert (not (= (re.* (str.to_re \"a\")) "
                        "(re.union (str.to_re \"\") "
                                    "(re.+ (str.to_re \"a\"))))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Substr unsat", "[sym-parikh-substr-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.* (str.to_re \"a\"))))\n"
        "(assert (= x (str.substr y 1 5)))\n"
        "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Substr sat", "[sym-parikh-substr-sat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.* (str.to_re \"a\"))))\n"
        "(assert (= x (str.substr y 1 5)))\n"
        "(assert (= x \"aa\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Prefix of unsat", "[sym-parikh-prefixof-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.* (str.to_re \"a\"))))\n"
        "(assert (str.prefixof x y))\n"
        "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Prefix of sat", "[sym-parikh-prefixof-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (str.prefixof x y))\n"
        "(assert (= x \"aa\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Bug is aab", "[sym-parikh-bug-is-aab]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (= y \"aab\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Suffix of unsat", "[sym-parikh-suffixof-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.* (str.to_re \"a\"))))\n"
        "(assert (str.suffixof x y))\n"
        "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Suffix of sat", "[sym-parikh-suffixof-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (str.suffixof x y))\n"
        "(assert (= x \"ab\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Contains unsat", "[sym-parikh-contains-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.* (str.to_re \"a\"))))\n"
        "(assert (str.contains y x))\n"
        "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Contains sat", "[sym-parikh-contains-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (str.contains y x))\n"
        "(assert (= x \"ab\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Replace unsat", "[sym-parikh-replace-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (= x (str.replace y \"b\" \"accc\")))\n"
        "(assert (= x \"aac\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}


TEST_CASE("Replace sat", "[sym-parikh-replace-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-const x String)\n"
        "(declare-const y String)\n"
        "(assert (str.in_re y (re.++ (re.* (str.to_re \"a\")) "
                                    "(str.to_re \"b\"))))\n"
        "(assert (= x (str.replace y \"b\" \"c\")))\n"
        "(assert (= x \"aac\"))\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Hex char sat", "[sym-parikh-hex-char-sat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re (_ char #x1a)))))\n"
        "(assert (= x (_ char #x1a)))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == sat);
}

TEST_CASE("Hex char unsat", "[sym-parikh-hex-char-unsat]") {
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
        "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
        "(assert (= x (_ char #x1a)))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat() == unsat);
}

TEST_CASE("Predicate limit applied", "[sym-parikh-pred-num-limit]") {
    std::shared_ptr<ParikhImageProvider> pip(new ParikhImageProviderOne());
    SymParikh sp = SymParikh::from_string(
        "(declare-fun x () String)\n"
            "(assert (str.in_re x (re.+ (str.to_re \"a\"))))\n"
            "(assert (str.in_re x (re.+ (str.to_re \"b\"))))\n"
            "(assert (str.in_re x (re.+ (str.to_re \"c\"))))\n"
            "(assert (str.in_re x (re.+ (str.to_re \"d\"))))\n"
            "(assert (str.in_re x (re.+ (str.to_re \"e\"))))\n"
            "(check-sat)\n",
        SymParikhConfig(pip, nullptr, false, false, 3)
    );

    // +1 for true
    REQUIRE(sp.get_predicates().size() == 4);
}

TEST_CASE("Word eq bug", "[sym-parikh-word-eq-bug]") {
    std::shared_ptr<ParikhImageProvider> pip(new ParikhImageProviderOne());
    SymParikh sp = SymParikh::from_string(
        "(declare-const x1 String)\n"
        "(assert (str.in_re x1 (re.range \"\\u{7c}\" \"\\u{e3}\")))\n"
        "(check-sat)\n"
    );
    REQUIRE(sp.check_parikh_sat());
}
